﻿using System.Collections.ObjectModel;
using ThinkGeo.MapSuite.Core;
using System;
using System.Collections;

namespace AdornmentInteractiveOverlaySample
{
    public class LabelAdornmentLayer : AdornmentLayer
    {
        private GeoFont font;
        private GeoColor color;
        private string labelText;

        public GeoFont Font
        {
            get { return font; }
            set { font = value; }
        }

        public GeoColor Color
        {
            get { return color; }
            set { color = value; }
        }

        public string LabelText
        {
            get { return labelText; }
            set { labelText = value; }
        }

        public LabelAdornmentLayer(string labelText, GeoFont font, int width, int height)
            : this(labelText, font, width, height, GeoColor.StandardColors.Black)
        {

        }

        public LabelAdornmentLayer(string labelText, GeoFont font, int width, int height, GeoColor color)
        {
            this.font = font;
            this.color = color;
            this.labelText = labelText;

            this.Width = width;
            this.Height = height;
        }

        protected override void DrawCore(GeoCanvas canvas, Collection<SimpleCandidate> labelsInAllLayers)
        {
            DrawingRectangleF textRect = canvas.MeasureText(labelText, font);
            ScreenPointF drawingLocation = GetDrawingLocation(canvas, (int)Width, (int)Height);

            String drawingText = labelText;

            //Compares the bounding box in screen coordinates with the size of the text.
            if (Width > textRect.Width)
            {
                //If fully within, it draws in one line.
                canvas.DrawText(
                drawingText,
                font,
                new GeoSolidBrush(color),
                new ScreenPointF[] { new ScreenPointF(drawingLocation.X + Width / 2, drawingLocation.Y + Height / 2) },
                DrawingLevel.LevelOne);
            }
            else
            {
                //If not, applies the word wrapping.
                string[] Words = Wrap(labelText, GetMaxStringLength(labelText, textRect.Width, Width));
                drawingText = "";
                for (int i = 0; i < Words.Length; i++)
                {
                    drawingText = drawingText + Words[i] + Environment.NewLine;
                }

                canvas.DrawText(
                 drawingText,
                 font,
                 new GeoSolidBrush(color),
                 new ScreenPointF[] { new ScreenPointF(drawingLocation.X + Width / 2, drawingLocation.Y + Height / 2) },
                 DrawingLevel.LevelOne);
            }

            textRect = canvas.MeasureText(drawingText, font);
            ActualWidth = textRect.Width;
            ActualHeight = textRect.Height;

            base.DrawCore(canvas, labelsInAllLayers);
        }

        //Function to get the maximum length of the string without having a new line.
        private int GetMaxStringLength(string text, float currentWidth, double boundingBoxScreenWidth)
        {
            //For now, the value is 10. You can write your own logic to calculate that value based on 
            // the size of the text and the bounding box to hold it.
            int result = (int)(text.Length * boundingBoxScreenWidth / currentWidth);
            if (result <= 0) { result = 1; }
            return result;
        }

        private string[] Wrap(string text, int maxLength)
        {
            string[] Words = text.Split(' ');
            int currentLineLength = 0;
            ArrayList Lines = new ArrayList(text.Length / maxLength);
            string currentLine = "";
            bool InTag = false;

            foreach (string currentWord in Words)
            {
                //ignore html
                if (currentWord.Length > 0)
                {

                    if (currentWord.Substring(0, 1) == "<")
                        InTag = true;

                    if (InTag)
                    {
                        //handle filenames inside html tags
                        if (currentLine.EndsWith("."))
                        {
                            currentLine += currentWord;
                        }
                        else
                            currentLine += " " + currentWord;

                        if (currentWord.IndexOf(">") > -1)
                            InTag = false;
                    }
                    else
                    {
                        if (currentLineLength + currentWord.Length + 1 < maxLength)
                        {
                            currentLine += " " + currentWord;
                            currentLineLength += (currentWord.Length + 1);
                        }
                        else
                        {
                            Lines.Add(currentLine);
                            currentLine = currentWord;
                            currentLineLength = currentWord.Length;
                        }
                    }
                }

            }
            if (currentLine != "")
                Lines.Add(currentLine);

            string[] textLinesStr = new string[Lines.Count];
            Lines.CopyTo(textLinesStr, 0);
            return textLinesStr;
        }
    }
}
